/* stdio.h: Standard input/output. */

#ifndef _STDIO_H
#define _STDIO_H

#include <stdarg.h>
#include <sys/types.h>

typedef struct
{
    int _fd;
    int _err;
    int _eof;
} FILE;

#define SEEK_SET 0
#define EOF -1

extern FILE* stderr;
#define stderr stderr

#ifdef __cplusplus
extern "C"
{
#endif

    int fflush(FILE*);

    /* Open a file and binds a stream to it. */
    FILE* fopen(const char* path, const char* mode);

    /* Close a file and frees up its stream. */
    int fclose(FILE* stream);

    /* Read arbitrarily sized items from a stream. */
    size_t fread(void* buf, size_t size, size_t nmemb, FILE* stream);

    /* Write arbitrarily sized items to a stream. */
    size_t fwrite(const void* buf, size_t size, size_t nmemb, FILE* stream);

    int fseek(FILE*, long, int);
    long ftell(FILE*);

    /* Return whether the error indicator was set in stream. */
    int ferror(FILE* stream);

    /* Return whether the end-of-file indicator was set in stream. */
    int feof(FILE* stream);

    /* Clear the error and end-of-file indicators in stream. */
    void clearerr(FILE* stream);

    void setbuf(FILE*, char*);

    int fprintf(FILE*, const char*, ...);
    int vfprintf(FILE*, const char*, va_list);

    /* Write formatted output into a buffer. */
    int sprintf(char* buf, const char* format, ...);

    /* Write up to max bytes of formatted output into a buffer. */
    int snprintf(char* buf, size_t max, const char* format, ...);

    /* Write formatted output into a buffer. */
    int vsprintf(char*, const char*, va_list);

    /* Write up to max bytes of formatted output into a buffer. */
    int vsnprintf(char*, size_t, const char*, va_list);

    /* Write formatted output to standard output. */
    int printf(const char*, ...);

    /* Write a string followed by a newline to standard output. */
    int puts(const char* s);

    /* Output a message to the console. */
    int console_write(const char* msg, size_t size);

    /* Write an error message to standard error. */
    void perror(const char* s);

#ifdef __cplusplus
}
#endif

#endif
